/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDENGINESERVICE_H
#define FEEDENGINESERVICE_H

#include "feedserviceshared.h"
#include "feedengineproxy.h"
#include "feedserviceutils.h"
#include "feederror.h"

class QDBusArgument;

class FeedEngineServicePrivate;

/*!
  \class FeedEngineService
  \brief The FeedEngineService component provides access to services provided
  by feed engine for the clients.
  The requests are made to feed engine in either sync or async manner. For asyn
  requests the client can connect to signals for the response and take necessary
  action when the request is completed. These signals are Qt signals and thus
  can be connected in Qt style.
  Some requests take more time than others. For this priority of all requests
  is not the same. It is also possible that the later request can be served before
  the previous one based on the priority implementation of feed engine.
  */
class FeedEngineService : public FeedEngineProxy
        , public Feeds::RestrictedCreation
{
Q_OBJECT

public:
    FeedEngineService();
    virtual ~FeedEngineService();

    //! Refresh list of channels.
    /*!
      Enqueue the list of channels as provided in uriList for refresh operation.
      The refresh operation is performed serially based on the details of channel
      represented by mfo:FeedChannel in tracker. For example the decision to
      use plugins is influenced by the plugin type for this channel for which
      request is made.
      */
    void refresh(const QList<QUrl> &uriList);
    //! Subscribe new feed channel.
    /*!
      New feed channel is added with the given links and pluginType.
      Returns true if addition in tracker is successful. The added channel is queued
      for download if network is available.
      */
    bool subscribe(const QList<QUrl> &link, const QString &pluginType);
    //! Unsubscribe the feed channel
    /*!
      Unsubscribes the feed channel by removing all the contents related to
      this channel including tracker data and enclosures downloaded for this
      channel.
      */
    bool unsubscribe(const QUrl &uri);
    //! Returns the last error encountered for the given list of channels
    /*!
      Error states are maintained by feed engine for all channels.
      Clients can interpret the errors referring to FeedError.
      Clients can pass an empty string list to get the error status for all channels
     */
    QVariantMap channelStatus(const QStringList &channelUriList);

    //! Returns the last error/state encountered for the given list of enclosures
    /*!
      Download state is maintained by feed engine for all enclosures.
      Clients can interpret the errors referring to FeedError.
      Clients can pass an empty string list to get the status for all enclosures,
      The result includes both pending enclosures and failed enclosures
     */
    QVariantMap enclosureStatus(const QStringList &enclosureUriList);

    //! Returns the current engine state
    /*!
      Returns the current state of engine.
      Result will indicate if engine is waiting on Mass storage mode, No-Network mode etc
     */
    int state();

    //! Returns transfer details for a specific transfer
    /*!
      Clients can request for transfer details like total size, downloaded size etc.
      for a specific transfer and feedengine will return those details.
     */
    QVariantMap transferDetails(const QString &transferId, const QStringList &fields);

    //! Cancel refresh for given channels.
    /*!
      Enqueued and current downloading channel is canceled for those channel
      uris present in uriList. The enclosure download for such a channel is also
      cleared if already present in feed engine queue.
      */
    void cancelRefresh(const QList<QUrl> &uriList);
    //! Edit already subscribed channel.
    /*!
      Successfully subscribed channels can be edited for some of the properties
      like title, link etc.
      @todo Document the properties parameter properly for clients
      */
    void editChannel(const QString &channelUri, const QVariantMap &properties);
    //! Download enclosure manually.
    /*!
      Manual download of enclosure can be trigerred using this call. If there is
      already some task being performed by feed engine, this download will be
      queued. The channelUri is uri of mfo:FeedChannel and enclosureUri is uri
      of mfo:Enclosure in tracker.
      */
    void downloadEnclosure(const QUrl &channelUri, QList<QUrl> &enclosureUriList);
    //! Cancel ongoing or queued enclosure download
    /*!
      Download is canceled for currently downloading enclosure. If the enclosure
      download is queued the queue is cleared for this enclosure.
      */
    void cancelDownloadEnclosure(const QUrl &channelUri, QList<QUrl> &enclosureUriList);    
    //! Pause ongoing or queued enclosure download
    /*!
      Download is paused for currently downloading enclosure. The remaining enclosures
      in the list will be cleared from the manual enclosure download queue.
      */
    void pauseDownloadEnclosure(const QUrl &channelUri, QList<QUrl> &enclosureUriList);
    //! Resume the paused enclosure download
    /*!
      Download is resumed for the first enclosure in the list. The remaining enclosures
      in the list will be enqueued to the manual enclosure download queue.
      */
    void resumeDownloadEnclosure(const QUrl &channelUri, QList<QUrl> &enclosureUriList);
    //! Enclosure download progress can be registered by clients.
    /*!
      Client can register for getting download progress for enclosure download
      for particular enclosure as given in enclosureUri
      */
    void registerEnclosureDownloadProgress(const QUrl &enclosureUri);
    //! Cancel getting download progress for the given enclosure
    /*!
      When client is no more interested in getting progress status of enclosure
      download it can cancel this. This is reverse of registerEnclosureDownloadProgress
      */
    void cancelEnclosureDownloadProgress(const QUrl &enclosureUri);

Q_SIGNALS:
    //! Sends edit channel status to client.
    /*!
      The result of edit of channel can be successful or not. This signal is used
      to communicate back to clients the result of edit operation.

      Typical connection of signals are done like following:

      \code
      #include <FeedServices>
      ...
      FeedServices *service = FeedServices::instance();
      QObject::connect(service->engine(), SIGNAL(editChannelStatus(QString,int)), this, SLOT(channelEdited(QString,int)));
      ...
      delete service;
      \endcode
      */
    void editChannelStatus(const QString &channelUri, int errorCode);
    //! Emitted when particular enclosure download is completed.
    /*!
      Indicates that enclosure download belonging to enclosureUri is completed.
      */
    void enclosureDownloadCompleted(const QString &enclosureUri, qlonglong timeElapsed);
    //! Error in downloading enclosures
    /*!
      List of enclosures represented in enclosureUriList for which common error
      was generated because of same fate.
      */
    void enclosureDownloadError(const QStringList &enclosureUriList, int errorCode, qlonglong timeElapsed);
    //! Sends enclosure download progress at regular interval
    /*!
      Enclosure progress status is sent to client at regular interval after a chunk of
      data is downloaded. The status consists of following parameters.
      \param enclosureUri Enclosure for which status is sent
      \param totalSize The size of enclosure currently downloaded (bytes)
      \param downloadedSize The size of enclosure downloaded so far (bytes)
      \param timeRemaining Approximate time remaining for download (seconds)
      */
    void enclosureDownloadProgress(const QString &enclosureUri, qlonglong totalSize, qlonglong downloadedSize, qlonglong timeRemaining);
    //! Emitted when particular enclosure download is started.
    /*!
      Indicates that enclosure download belonging to enclosureUri is started.
      */
    void enclosureDownloadStarted(const QString &enclosureUri);
    //! Emitted when particular channel download is completed.
    /*!
      Indicates that refresh of a particular channel represented by channelUri is
      completed successfully.
      */
    void refreshCompleted(const QString &channelUri);
    //! Updates the clients about the change in engine state
    /*!
      When device enters mass storage mode or no-network state, clients are informed with the state change.
      Even failures like refresh failure are informed with this signal.
      */
    void refreshState(int state);

    //! When a particular channel download fails.
    /*!
      The channel belonging to channelUri fails to download because of error
      stated in errorCode.
      */
    void refreshError(const QString &channelUri, int errorCode);
    //! Emitted when refresh of channel is started
    /*!
      Download of channel belonging to channelUri is started.
      */
    void refreshStarted(const QString &channelUri);

private:
    Q_DISABLE_COPY(FeedEngineService);
    Q_DECLARE_PRIVATE(FeedEngineService);
    FeedEngineServicePrivate * const d_ptr;
};

QDBusArgument &operator<<(QDBusArgument &out, const ErrorCode &);
const QDBusArgument &operator>>(const QDBusArgument &in, ErrorCode &);


#endif // FEEDENGINESERVICE_H
