/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDSERVICES_H
#define FEEDSERVICES_H

#include <QtGlobal>

class FeedEngineService;
class FeedSettingService;
class FeedServicesPrivate;

/*!
  \class FeedServices
  \brief The FeedServices is utility class for using services provided by feed engine.
  FeedServices provides components from which related methods can be called.
  For using any component feed services instance needs to be called only once.
  Feed services take care of initializing and de-initializing of these compoents.

  A typical flow for using any component will be like following:

  \code
  #include <FeedServices>
  ...
  FeedServices *service = FeedServices::instance();
  // Use engine component services
  service->engine()->downloadEnclosure(channelUri, enclosureUriList);
  // Use settings component services
  QVariantMap settings = service->settings()->conf();
  delete service;
  ...
  \endcode

  Refer to FeedEngineService and FeedSettingService to know
  about kind of services provided by them in detail.
  */
class FeedServices
{
public:
    //! Creates and return the FeedServices static instance.
    /*!
      After the services are used, this instance must be deleted by caller.
      */
    static FeedServices* instance();
    virtual ~FeedServices();

    //! To use services of engine component. For details see FeedEngineService
    /*!
      The ownership and hence deletion of this object is handled by FeedServices.
      So no need to delete this explicitly.
      */
    FeedEngineService* engine();
    //! To use services of settings of feed engine. For details see FeedSettingService
    /*!
      The ownership and hence deletion of this object is handled by FeedServices.
      So no need to delete this explicitly.
      */
    FeedSettingService* settings();
private:
    FeedServices();
    Q_DISABLE_COPY(FeedServices);
    Q_DECLARE_PRIVATE(FeedServices);
    FeedServicesPrivate * const d_ptr;
    static FeedServices *onlyInstance;
};


#endif // FEEDSERVICES_H
