/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDCHANNEL_H
#define FEEDCHANNEL_H

#include "feednode.h"
#include "feedmessage.h"
#include <QtCore/QMetaType>
#include <QtCore/QUrl>

class QDebug;
class QString;
class FeedChannelPrivate;
class FeedEnclosure;

/*!
\class FeedChannel
\brief Channel data structure that is used in feedengine for communication with plugin.

It is the top level of information in feeds. Each feed channel can have many FeedMessage.

All feed readers subscribe to these channel. A website usually provide one channel for subscription
in RSS/ATOM or any other format. Some website may provide more than one channel. Notable example
is youtube's channel for: Top Favorites, Top rated etc

The channel data retrived from web is specific to format used. Plugin's job is to convert that data
to FeedChannel structure. Some helper classes can be used for this purpose provided by this package.
*/
class FeedChannel : public FeedNode
{
    //! For printing debug messages like qDebug() << FeedChannel
    friend QDebug operator<<(QDebug dbg, const FeedChannel & channel);

    //! Writes the given <i>channel</i> to the specified <i>stream</i>
    friend QDataStream& operator<< ( QDataStream & stream, const FeedChannel& channel );

    //! Reads a FeedChannel from the specified <i>stream</i> into the given <i>channel</i>
    friend QDataStream& operator>> ( QDataStream & stream, FeedChannel & channel );

public:
    FeedChannel();

    ~FeedChannel();

    //! Constructs a copy of other.
    FeedChannel(const FeedChannel & other);

    //! Constructor
    FeedChannel(const QString &uri, const QString &link,
        const QString &title, const FeedEnclosure &icon, const QString &type,
        const QString &description, const QDateTime &subscribedTime, const QDateTime &updatedTime, const QString &lastModifiedTime, const QList<FeedMessage> msgList);

    //! Assigns other to this FeedChannel and returns a reference to this FeedChannel
    FeedChannel& operator=(const FeedChannel & other);

    //! Returns the channel uri
    QString uri() const;

    //! Sets the channel uri
    /*!
      Channel uri is unique identification key. Also used as key in tracker database.
      */
    void setUri(const QString &uri);

    //! Returns the channel link
    QString link() const;

    //! Sets the link of channel
    /*!
      This is the weblink from where feeds content is downloaded
      */
    void setLink(const QString &link);

    //! Returns type of channel
    QString type() const;

    //! Sets type of channel
    /*!
      This is plugin specific data. Used by engine to invoke suitable plugin.
      For example for RSS/ATOM feeds RSSATOM plugin is invoked.
      In this case plugin type is "rssatom"
      */
    void setType(const QString &type);

    //! Returns the channel icon.
    FeedEnclosure icon() const;

    //! Sets the channel icon
    /*!
      Channels usually published channel icon comparable to thumbnails.
      All the media type are mapped to FeedEnclosure in feeds structures.
      Hence the channel icon is also of type FeedEnclosure
      */
    void setIcon(const FeedEnclosure &icon);

    //! Returns title of the channel
    QString title() const;

    //! Sets title of the channel
    void setTitle(const QString &title);

    //! Returns description of the channel
    QString description() const;

    //! Sets description of the channel
    void setDescription(const QString &description);

    //! Returns subscribed time of the channel
    QDateTime subscribedTime() const;

    //! Sets subscribed time of the channel
    void setSubscribedTime(const QDateTime &subscribedTime);

    //! Returns updated time of the channel
    QDateTime updatedTime() const;

    //! Sets updated time of the channel
    void setUpdatedTime(const QDateTime &updatedTime);

    //! Returns last modified time of the channel
    QString lastModifiedTime() const;

    //! Sets last modified time of the channel
    void setLastModifiedTime(const QString &time);

    //! Adds child node FeedMessage to this FeedChannel
    void addChild(FeedNode *);

    //! Returns list of FeedMessage belonging to this channel
    QList<FeedMessage> messages() const;

    //! Sets the message list to this channel
    void setMessages(QList<FeedMessage> &msgList);

private:
    void insertMessage(const FeedMessage &msg);
    void detach();
    FeedChannelPrivate *d;
};

Q_DECLARE_METATYPE(FeedChannel)

QDataStream& operator<< ( QDataStream& stream, const FeedChannel& channel );
QDataStream& operator>> ( QDataStream& stream, FeedChannel& channel );

#endif // FEEDCHANNEL_H
