/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDENCLOSURE_H
#define FEEDENCLOSURE_H

#include "feednode.h"
#include <QtCore/QMetaType>

class QDebug;
class QString;
class FeedEnclosurePrivate;

/*!
\class FeedEnclosure
\brief Enclosure data structure that is used in feedengine for communication with plugin
It is the third level of information in feeds next to FeedMessage. Each FeedMessage
contains multiple FeedEnclosures.

ATOM 1.0 specification it corresponds to tag 'atom:link rel="enclosure"'
RSS 2.0 specification it corresponds to tag 'enclosure'
Yahoo Media RSS specification it corresponds to <media:content>
*/
class FeedEnclosure : public FeedNode
{
    //! For printing debug messages like qDebug() << FeedEnclosure
    friend QDebug operator<<(QDebug dbg, const FeedEnclosure & enclosure);

    //! Writes the given <i>enclosure</i> to the specified <i>stream</i>
    friend QDataStream& operator<< ( QDataStream & stream, const FeedEnclosure & enclosure );

    //! Reads a FeedEnclosure from the specified <i>stream</i> into the given <i>enclosure</i>
    friend QDataStream& operator>> ( QDataStream& stream, FeedEnclosure & enclosure );

public:
    
    //! \enum Type
    /*!
      \brief Enclosure type.

      <b>Essentail enclosures</b> are always downloaded along with feed messages as they
      are considered integral part of feed message.
      <b>Optional enclosures</b> are downloaded if settings allowed to do so. These are also
      downloaded manually by user.
      */
    enum Type
    {
        EssentialEnclosure = 1,
        OptionalEnclosure  = 2
                         };

    FeedEnclosure();
    ~FeedEnclosure();

    //! Constructs a copy of other.
    FeedEnclosure(const FeedEnclosure & other);

    //! Assigns other to this FeedEnclosure and returns a reference to this FeedEnclosure.
    FeedEnclosure& operator=(const FeedEnclosure & other);

    //! Returns true if string other is equal to this string; otherwise returns false.
    bool operator==(const FeedEnclosure & other);

    //! Constructs FeedEnclosure object
    FeedEnclosure(const QString &uri, const QString &remoteUrl, const QString &localUrl,
                  const QString &sourceNamespaceUri, quint32 length, Type type, quint32 groupId, bool isDefault);

    //! Returns the enclosure uri.
    QString uri() const;

    //! Sets the enclosure uri
    /*!
      Enclosure uri is unique identification key. Also used as key in tracker database.
      */
    void setUri(const QString &uri);

    //! Returns remote Url of the enclosure
    QString remoteUrl() const;

    //! Sets the remote Url of the enclosure
    /*!
      Remote url is the link from where enclosure is downloaded over web
      */
    void setRemoteUrl(const QString &url);

    //! Returns the local Url of the enclosure
    QString localUrl() const;

    //! Sets the local Url of the enclosure
    /*!
      Local url is the file url of the downloaded enclosure if it is downloaded.
      */
    void setLocalUrl(const QString &url);

    //! Returns the namespace for this enclosure
    QString sourceNamespace() const;

    //! Sets the source namespace for this enclosure
    void setsourceNamespace(const QString &namespaceUri);

    //! Returns the lenght (size) of this enclosure in bytes
    quint32 length() const;

    //! Sets the lenght (size) of this enclosure in bytes
    void setLength(quint32 length);

    //! Returns the type of this enclosure
    Type type() const;

    //! Sets the type of this enclosure
    void setType(Type type);

    //! Returns the group id of this enclosure
    quint32 groupId() const;

    //! Sets the group id of this enclosure
    /*!
      Some feeds specs like Media RSS supports groups for enclosures.
      In those case there can be more than one enclosure in different format
      or language. Each enclosure in such a group is given a unique group id.
      */
    void setGroupId(quint32 groupId);

    //! Whether the given enclosure is default enclosure in enclosure group
    bool isDefault() const;

    //! Sets the default nature of this enclosure
    /*!
      In group of enclosures, one enclosure is usually default enclosure.
      */
    void setDefault(bool isDefault);

    //! Returns mime type of this enclosure
    QString mimeType() const;

    //! Sets mime type of this enclosure
    void setMimeType(const QString &mimeType);

    //! Returns enclosure type icon of this enclosure
    QString typeIcon() const;

    //! Sets enclosure type icon of this enclosure
    void setTypeIcon(const QString &icon);

    //! Does not do anything as of now, as it is lowest in parent child hierarchy
    void addChild(FeedNode *);

private:
    void detach();
    FeedEnclosurePrivate *d;
};

Q_DECLARE_METATYPE(FeedEnclosure)
Q_DECLARE_METATYPE(QList<FeedEnclosure>)

QDataStream& operator<< ( QDataStream& stream, const FeedEnclosure& enclosure );
QDataStream& operator>> ( QDataStream& stream, FeedEnclosure& enclosure );

#endif // FEEDENCLOSURE_H
