/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDMESSAGE_H
#define FEEDMESSAGE_H

#include "feednode.h"
#include <QtCore/QMetaType>

class QDebug;
class QString;
class QDateTime;
class QUrl;
class FeedMessagePrivate;
class FeedEnclosure;

/*!
\class FeedMessage
\brief Message data structure that is used in feedengine for communication with plugin
It is the second level of information in feeds next to FeedChannel. Each FeedChannel
contains multiple FeedMessage. For example it can be post for ATOM/RSS xml or episode for a
podcast.

ATOM 1.0 specification it corresponds to "atom:entry" element
RSS 2.0 specification it corresponds to 'item' tag

However it may be noted that the mapping of such items is prerogative of the plugin writer.
Plugins can also take advantage of this by dumping some data which is not understood by
feedengine in some FeedMessage property like description. Then clients may read the description
and does suitable translation before consuming it. This can be used to extend functionality to
case other than standard RSS/ATOM type xml.
*/
class FeedMessage : public FeedNode
{
    //! For printing debug messages like qDebug() << FeedMessage;
    friend QDebug operator<<(QDebug dbg, const FeedMessage &message);

    //! Writes the given <i>message</i> to the specified <i>stream</i>
    friend QDataStream& operator<< ( QDataStream& stream, const FeedMessage& message );

    //! Reads a FeedMessage from the specified <i>stream</i> into the given <i>message</i>
    friend QDataStream& operator>> ( QDataStream& stream, FeedMessage& message );

public:
    FeedMessage();
    ~FeedMessage();

    //! Overloaded constructor
    FeedMessage(const QString &title, const QString &link,const QString &desc, const QDateTime &downloadedTime);

    //! Overloaded constructor
    FeedMessage(const QString &title, const QString &link, const QString &id,const QString &desc, const QString &summary,
                const QDateTime &publishedDate, const QList<FeedEnclosure> &enclosures,
                const QDateTime &downloadedTime, bool favorite);

    //! Constructs a copy of other.
    FeedMessage(const FeedMessage & other);

    //! Assigns other to this FeedMessage and returns a reference to this FeedMessage.
    FeedMessage& operator=(const FeedMessage & other);

    //! Returns title of FeedMessage
    QString title() const;

    //! Sets the title of FeedMessage
    void setTitle(const QString &title);

    //! Returns link of FeedMessage
    QString link() const;

    //! Sets the link of FeedMessage
    void setLink(const QString &link);

    //! Returns id of the message
    QString id() const;

    //! Sets the id for this message
    /*!
      Some feeds specifications like ATOM provide id for messages
      */
    void setId(const QString &id);

    //! Returns the desciption of the message
    QString description() const;

    //! Sets the description of the message
    void setDescription(const QString &description);

    //! Returns the summary for the message
    QString summary() const;

    //! Sets the summary for the message
    /*!
      The summary of the message is built from description. Sometimes it makes
      more sense to show summary to the user before showing description.
      */
    void setSummary(const QString &summary);

    //! This is used for setting base url for embedded urls in feed.
    //! \todo Shift this to pimpl
    void setBaseUrl(const QUrl &baseUrl);

    //! Returns the base url
    QUrl baseUrl() const;

    //! Returns the publishedDate of feeds
    QDateTime publishedDate() const;

    //! Set the published date of feeds
    void setPublishedDate(const QDateTime &pubDate);

    //! Returns the downloaded time of feeds
    QDateTime downloadedTime() const;

    //! Sets the downloaded time of feeds
    void setDownloadedTime(const QDateTime &downloadedTime);

    //! Returns the favorite status of feed message
    bool isFavorite() const;

    //! Sets the favorite status of feed message
    void setFavorite(bool favorite);

    //! Returns the list of enclosures which belongs to this message
    QList<FeedEnclosure> enclosures() const;

    //! Set method for message's enclosures
    /*!
      Clears existing list if any, and create new list of enclosures
      */
    void setEnclosures(const QList<FeedEnclosure> enclosures);
    
    //! Adds list of enclosures to the current enclosure list
    void addEnclosure(const QList<FeedEnclosure> &enclosureList);

    //! Adds a single enclsoure to the current enclosure list
    void addEnclosure(const FeedEnclosure &enclosure);
    
    //! Adds child node FeedEnclosure to this FeedMessage
    void addChild(FeedNode *);

private:
    void detach();
    FeedMessagePrivate *d;
};

Q_DECLARE_METATYPE(FeedMessage)
Q_DECLARE_METATYPE(QList<FeedMessage>)

QDataStream& operator<< ( QDataStream& stream, const FeedMessage& message );
QDataStream& operator>> ( QDataStream& stream, FeedMessage& message );

#endif // FEEDMESSAGE_H
