/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDPLUGININTERFACE_H
#define FEEDPLUGININTERFACE_H

#include <QObject>
#include <QVariantMap>

class FeedChannel;
class FeedEnclosure;

//! Identify interface version
static const QString FEED_PLUGIN_INTERFACE_VERSION("0.1");

/*!
  \class FeedPluginInterface
  \brief Provides abstract interface for plugins.
  Plugins must implement abstract methods by subclassing this class.
  There are two kinds of methods that plugins need to support. Those are
  async and sync methods respectively.

  <b>Asynchronous methods</b> are used for giving time consuming task (network download).
  The response mechanism for such methods is based on Qt signals which are required to
  be emitted by plugin. Feed engine can decide to unload a plugin based on various criterion,
  one of which is non responsiveness. Hence it is imperative for plugins to emit signal as early
  as job is done.

  <b>Synchronous methods</b> are called to collect the data from plugins. After the job is done by
  plugin, plugin sends signal to engine. Depending on state of engine and the signal(error or success),
  feedengine asks for data in form of FeedChannel structure. FeedChannel structure has nested data
  structures with list of FeedMessage structure.
  */
class FeedPluginInterface : public QObject
{
    Q_OBJECT
public:
    //! Constructor
    FeedPluginInterface(QObject* parent = 0);

    //! Destructor
    virtual ~FeedPluginInterface() {}

    //! Download the feeds
    /*! Asynchronous call for downloading feed content. The details of feeds to be
      * downloaded like type, remote url etc is mentioned in FeedChannel
      * Signal needs to be emitted when download is completed.
      * \param channel which contains all the details of feed to be downloaded
      */
    virtual void downloadFeeds(const FeedChannel &channel) = 0;

    //! Download the given enclosure
    /*! Download progress signal as well as completed signal needs to be emitted by plugin.
      * \param enclosure object which contains the remote URL to download
      */
    virtual void downloadEnclosure(const FeedEnclosure &enclosure) = 0;

    //! Cancel ongoing download. It will cancel current feed or enclosure download in progress
    /*! At a time either feed or enclosure download is in progress. When cancel is called it should
      be canceled.
      */
    virtual void cancelDownload() = 0;

    //! Pause ongoing download. It will pause the enclosure download in progress
    virtual void pauseDownloadEnclosure() = 0;

    /*! Resume the download of the given enclosure
      * \param enclosure object which contains the remote URL to download
      */
    virtual void resumeDownloadEnclosure(const FeedEnclosure & enclosure) = 0;

    //! Get the feed channels
    /*! This is called by feedengine after download of feed is completed.
      Plugins convert the downloaded data to FeedChannel structure.
      */
    virtual FeedChannel channel() const = 0;

private slots:
    //! Return version of interface
    QString version() const;

Q_SIGNALS:
    //! This is a signal which is emitted when the download and parsing of the requested feed is done successfully.
    void downloadFeedsFinished();

    /*! This signal is emitted when download or parsing of the requested feed fails
     \param error contains the error reason
    */
    void downloadFeedsFailed(int error);

    //! This is a signal which is emitted when the downloading of enclosures is done successfully.
    void downloadEnclosureFinished(QVariantMap map);

    /*! This signal is emitted when downloading of enclosures fail.
     \param error contains the error reason
     */
    void downloadEnclosureFailed(int error);

    /*! This is a signal to indicate download progress.
      *
      * \param bytesReceived new data received in terms of bytes
      * \param bytesTotal total bytes to be downloaded
      */
    void enclosureDownloadProgress(  qint64 bytesReceived, qint64 bytesTotal );
};


#endif // FEEDPLUGININTERFACE_H
