/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDTRACKERSPARQL_H
#define FEEDTRACKERSPARQL_H

#include <QObject>
#include <QVariantMap>
class QStringList;
class QUrl;
class QDateTime;

class FeedChannel;
class FeedMessage;
class FeedEnclosure;
class QSparqlConnection;

/*!
  \class FeedTrackerSparql
  \brief The FeedTrackerSparql provides a common interface for querying and
  saving tracker data related to feeds ontology.
  This class uses tracker sparql direct driver for all its operations.
  */
class FeedTrackerSparql : public QObject
{
 Q_OBJECT

public:

    enum ChannelProperty
    {
       Link                             = 1,
       Title                             = 2,
        LastModifiedTime   = 3,
       Invalid                         = 4
    };

    //! static method used to get instance of FeetTracker
    static FeedTrackerSparql* instance();

    //! static method used to get instance of FeetTracker
    static void releaseInstance();

    //! Destructor
    ~FeedTrackerSparql();

    //! Methods:
    //! Returns all feed channels in tracker
    QList<FeedChannel> allChannels() const;

    //! Return channel for this particular uri
    FeedChannel channelForUri(const QString &uri) const;

    //! Returns uri of all channels in tracker
    QStringList allChannelUrls() const;

    //! Returns enclosure for this partucular uri
    FeedEnclosure enclosureForUri(const QString &uri) const;

    //! Returns channel type for this particular channel
    QString channelType(const QString &channelUri) const;

    //! Delete all channels. Effectively deleting all its messages, enclosures, image etc
    void deleteAllChannels();

    //! To Get Deletetable Message Count
    int deletableMessageCount(const QString &channelUri) const;

    //! To Get List of Message Uris which can be deleted
    QStringList deletableMessages(const QString &channelUri, int limit);

    //! To get LastestPostPublishedTime of messages in a channel
    QDateTime latestPostPublishedTime(const QString &channelUri);

    //! To Get List of Message Links for a given Channel
    QStringList messageLinksOfChannel(const QString &channelUri) const;

    //! To Get all message Uris
    QStringList allMessageUrls(const QString &channelUri) const;

    //! Get List of FeedMessages in a channel
    QList<FeedMessage> messagesForChannel( const QString &channelUri) const;

    //! Delete channel image
    void deleteChannelImage(const QString &channelUri) const;

    //! Method to add a feed task to tracker
    QString addChannel(const FeedChannel &channel) const;

    //! Method to update a feed task to tracker
    bool updateChannel(const FeedChannel &channel) const;

    //! To Modify a Channel
    bool editChannel(const QString &uri, const QVariantMap &map) const;

    //! To Update Enclosure Information
    bool updateEnclosure(const FeedEnclosure &enclosure) const;

    //! To Delete a Message of given Channel
    void deleteMessage(const QString &messageUri, const QString &channelUri);

    //! Delete all Feed Enclosures for a given message
    void deleteEnclosures(const QString &messageUri);

    //! Delete a specific feed channel
    void deleteChannel(const QString &channelUri);

    //! Method to delete all feed entries corresponding to feed task
    bool deleteMessages(const QString &channelUri);

    //! To add list of feedmessages to tracker and have logical relation with channel having channelUri
    bool addMessages(const QList<FeedMessage> &messages, const QString &channelUri) const;

    //! To add list of enclosures to tracker and have logical relation with message having messageUri
    bool addEnclosures(const QList<FeedEnclosure> &enclosures, const QString &messageUri) const;

    //! Get List of FeedEnclosures for a message
    QList<FeedEnclosure> enclosuresForMessage(const QString &messageUri) const;

    //! To Delete Orphan Nodes from tracker
    void deleteOrphans();


#ifdef FEEDUTILITY_TEST
    void deleteEnclosure(const QString &enclosureuri) const;
#endif

signals:
    void enclosuresToBeDeleted(QList<FeedEnclosure>& enclosureList);

private:
    void deleteMessages(const QList<QUrl> messageUrls);
    void deleteEnclosures(const QList<QUrl> enclosureUrls);
   QString addMessageQuery(const FeedMessage &message, const QString &channelUri) const;
   void deleteOrphanMessages();
   void deleteOrphanEnclosures();

private:
    //! Constructor
    FeedTrackerSparql();
    QSparqlConnection *_feedQTrackerDirectConnection;
    static FeedTrackerSparql *_single;
    static const QString    TitleKey;
    static const QString    LinkKey;
    static const QString    LastModifiedTimeKey;
};

#endif
