/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef QST_LOGGER_H
#define QST_LOGGER_H

#include <QtDebug>

// define QST_ENABLE_LOGGER to enable logging
//#define QST_ENABLE_LOGGER

#ifdef QST_ENABLE_LOGGER

#include "qstglobal.h"
#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QHash>
#include <QtCore/QFile>
#include <QtCore/QTextStream>

class QSettings;

class QstLogger : public QObject
{
    Q_OBJECT
    Q_ENUMS(Status)
public:
            enum Status {
        NoError = 0,
        ConfigAccessFailed,
        InvalidSettings
    };

    struct CategorySettings
    {
        QString _name;
        char    _debugEnabled       :1;
        char    _warningEnabled     :1;
        char    _criticalEnabled    :1;
        char    _fatalEnabled       :1;
        char    _unused             :4;
    };
    struct LogInfo
    {
        QtMsgType msgType;
        QString category;
        QString file;
        QString line;
        QString longFunction;
        QString shortFunction;
        QString message;
    };

    QstLogger(const QString &configFile);
    ~QstLogger();
    static QstLogger* instance();
    Status status() const;
    bool log(const LogInfo logInfo);
    CategorySettings settingsForName(const QString &name) const;

public slots:
    bool loadSettings();

private:
    struct Field
    {
        bool logged;
        bool showFull;
        short  length;

        Field():logged(false), showFull(false), length(0) { }
    };

    bool extractLogSettings(QSettings &settings);
    void extractLogFormat(QSettings &settings);
    void extractFieldSettings(QSettings &settings,
                              const QString &fieldName, Field &field);

    void registerCategory(const CategorySettings &);

    QString timestampField();
    QString fileField( const QString &filePath);
    QString functionField( const QString &longName, const QString &shortName);
    QString lineField( const QString &value);
    QString msgTypeField(QtMsgType msgType);

private:
    static QstLogger *_self;
    QHash<QString, CategorySettings> _categories;
    QString      _confFilePath;
    QString      _logFilePath;
    QFile        _logFile;
    QTextStream  _logStream;

    QString _logFormat;
    Field   _timeField;
    Field   _fileField;
    Field   _lineField;
    Field   _msgTypeField;
    Field   _functionField;
    Field   _messageField;
    Status  _status;
    bool    _logToConsole;
    bool    _logsEnabled;
};

#define QST_LOGGER_INIT(var, conf) QstLogger var(conf)

#define QST_LOGGER_DELIMTER  QString("\n")

// category\nfile\nline\nlong-function\nshort-function\n

#define QST_LOGGER_STREAM(stream, category) \
stream << qPrintable( \
#category   + QST_LOGGER_DELIMTER \
                      + __FILE__  + QST_LOGGER_DELIMTER \
                      + QString::number(__LINE__) + QST_LOGGER_DELIMTER \
                      + QST_LONG_FUNCTION  + QST_LOGGER_DELIMTER \
                      + QST_SHORT_FUNCTION + QST_LOGGER_DELIMTER)

#define qstLogDebug(category)    QST_LOGGER_STREAM(qDebug(),    category)
#define qstLogWarning(category)  QST_LOGGER_STREAM(qWarning(),  category)
#define qstLogCritical(category) QST_LOGGER_STREAM(qCritical(), category)
#define qstLogFatal(category)    QST_LOGGER_STREAM(qFatal(),    category)

#else
#define QST_LOGGER_INIT(var, conf) //
        // logger disabled
#define qstLogDebug(category)    if(true); else qDebug()
#define qstLogWarning(category)  if(true); else qDebug()
#define qstLogCritical(category) if(true); else qDebug()
#define qstLogFatal(category)    if(true); else qDebug()
#endif


#endif // QST_LOGGER_H
