/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


#ifndef FEEDABSTRACTXMLCONVERTER_H
#define FEEDABSTRACTXMLCONVERTER_H

#include "feednode.h"
#include <FeedNamespacePluginManager>
#include <QObject>
#include <QDomElement>
#include <QVariant>

//! Abstract class for all Xml Converters. All converters need to be derived from this
//! This class also contains a few common functions to be used by all XmlConverters.
class FeedAbstractXmlConverter : public QObject
{
   Q_OBJECT

public:
    FeedAbstractXmlConverter();    
    virtual ~FeedAbstractXmlConverter();

    //! Interface to parse a DOM tree. Tree could be a channel/message/enclosuregroup/enclosure subtrees from a DOM document
    virtual bool parseDomNode(const QDomElement &node) = 0;
    //! Each node must return the FeedNode it holds. This will be used to set the child nodes for this node.
    //! Eg: A channel FeedNode will set as parent to a message node.
    virtual FeedNode* feedNode() = 0;

    //! Sets the parent Xml Converter which created this Xml Converter.
    //! Eg: FeedChannelXmlConverter object will be set for a FeedMessageXmlConverter.
    void setParentXmlConverter(FeedAbstractXmlConverter *parentXmlConverter);

    //! Sets the Namespace Plugin manager to be used by the converter
    void setNamespacePluginManager(FeedNamespacePluginManager *namespaceManager);

    //! Returns the value of a given Meta property. Making this public so that FeedXmlConverter can query baseUrl of channel
    QVariant propertyValue(const QString &propertyName);

protected: //data. We can avoid providing a method for each member here by making them protected
    //! FeedNamespacePluginManager object to be used to handle the namespace nodes
    FeedNamespacePluginManager *_namespacePluginManager;

protected: // methods. These are generic methods which could be used by all converters

    //! Extracts the base URL from root node
    void extractBaseUrl(const QDomElement &root);

    //! Stores a given pair of <name, value> Meta property
    bool setProperty(const QString &name, const QVariant &value);

    //! Extracts Namespace node info by using the  _namespacePluginManager
    QDomElement extractNamespaceNode(const QDomElement &node);

    //! Adds a a given FeedNode object as a child to the parent FeedNode object
    bool addFeedNode(FeedNode *node);

    //! Returns the parent node. This is especially useful when a Xml Converter does own any object.
    //! It can fetch the FeedNode of parent xml converter.
    //! Eg: FeedEnclosureGroupXmlConverter uses this to set an enclosure found in its scope to the message, to which the enclosuregroupo belongs to.
    FeedNode* feedParentNode();

private: // data.
    FeedAbstractXmlConverter *_parentXmlConverter;   

};

#endif // FEEDABSTRACTXMLCONVERTER_H
