/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/


/*!
 ** feeddownloadmanager.h
 ** Provides interfaces to download feed task
 **/

#ifndef FEEDDOWNLOADMANAGER_H
#define FEEDDOWNLOADMANAGER_H

#include <QObject>
#include <QUrl>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QSslError>

/*!
  \class FeedDownloadManager helper class that accepts a link to download from plugin and returns the downloaded content.
 */
class FeedDownloadManager: public QObject
{
    Q_OBJECT
public:
    //! constructor
    FeedDownloadManager(QObject *parent = 0);

    /*! This function initiates download of a given URL provided by plugin.
    * \param url URL to download.
    * \param device IO Device to write. This could be a QString or a file and passed by the plugin
    */
    void start(const QUrl &url, QIODevice *device);

    //! Terminate ongoing download
    void stop();

    //! Pause ongoing download
    void pause();

    //! Resume paused download
    void resume(const QUrl &url, QIODevice *device);

    /*! This function is used to set Accept Header by plugin
      * \param acceptHeader Accept String
      */
    void setAcceptHeader(QString acceptHeader);

    /*! This function is used to set custom HTTP Header by plugin
      * \param headerName  Header Name
      * \param header  Header value
      */
    void setRawHeader( const QByteArray& headerName, const QByteArray& header );

signals:

    /*! This is a signal to indicate the requested download is complete
    * \param reply Response to the downloaded request
    */
    void finished(QNetworkReply *reply);

    /*! This is a signal to indicate the requested download is paused
    */
    void paused();

    /*! This is a signal to indicate download progress.
      *
      * \param bytesReceived new data received in terms of bytes
      * \param bytesTotal total bytes to be downloaded
      */
    void progress( qint64 bytesReceived, qint64 bytesTotal );

private slots:

    /*! This is a slot which is called when the download is complete
    */
    void downloadFinished();

    /*! This is a slot which is called when part of the download data is received and ready to read.
      * It just appends the data to the IODevice which is passed by plugin in downloadUrl() call.
    */
    void downloadReadyRead();

    /*! This is a slot receive download progress from network access manager
     *
     * \param bytesReceived new data received in terms of bytes
     * \param bytesTotal total bytes to be downloaded
     */
    void slotDownloadProgress( qint64 bytesReceived, qint64 bytesTotal );

    //! SSL errors
    void slotSslErrors(const QList<QSslError> & errors);

private: // functions

    //! start actual download process
    void download();

    //! Extracts proxy details
    void setProxyDetails();

    //! handles the redirecion
    void handleRedirection( const QUrl& redirectionUrl );

private:

    enum DownloadState
    {
        Idle,
        Downloading,
        DownloadPaused
    };

private:
    //! QNetworkAccessManager object used to place a request to the network
    QNetworkAccessManager _manager;

    //! Used by _manager to indicate the status of download
    QNetworkReply *_currentDownload;

    //! Current download request
    QNetworkRequest _currentRequest;

    //! IODevice where the downloaded data is written.
    QIODevice *_device;

    //! Download size when download paused, required to calculate correct progress
    qint64 _downloadSizeAtPause;

    qint64 _previousDownloadProgress;

    //! Current state of download manager
    DownloadState _downloadState;
};

#endif

