/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Alexander Bokovoy (alexander.bokovoy@nokia.com)
**
** This file is part of Feed engine.
**
** $FEED_ENGINE_BEGIN_LICENSE$
** This software, including documentation, is protected by copyright controlled by
** Nokia Corporation. All rights are reserved. Copying, including reproducing, storing,
** adapting or translating, any or all of this material requires the prior written consent
** of Nokia Corporation. This material also contains confidential information which may
** not be disclosed to others without the prior written consent of Nokia.
** $FEED_ENGINE_END_LICENSE$
**
****************************************************************************/

#ifndef FEEDDOWNLOADDIALOG_H
#define FEEDDOWNLOADDIALOG_H

#include <MDialog>

class MImageWidget;
class FeedDownloadDialogPrivate;

class FeedDownloadDialog : public MDialog
{
Q_OBJECT
Q_ENUMS (FeedDownloadState);
public:
    enum FeedDownloadState  {
        None = 0,
        Idle,         // Idle state can be due to Paused or Pending states.
        Progressing,  // Currently downloading.
        Failed,       // Download failed.
        Completed     // Download completed.
    };

    FeedDownloadDialog();
    virtual ~FeedDownloadDialog();

    /*!
      Used to set the icon for the download dialog. In case icon starts with "/"
      it is considered as path to a thumbnail. Else it will be considered as
      an icon ID.
     */
    void setIcon(const QString &icon);

    /*!
      Name of the file that is currently being downloaded.
     */
    void setFileName(const QString &fileName);

    /*!
      Source from where the file is being downloaded. It can be feed name or
      an URL.
     */
    void setFileSource(const QString &fileSource);

    /*!
      Size of the file currently being downloaded.
     */
    void setFileSize(long fileSize);

    /*!
      Shows the download progress. This replaces the text message shown when the
      download is not happening. In case there is a need to show indefinite progress
      then set all parameters to 0.
     */
    void showProgress(qlonglong totalSize, qlonglong downloadedSize, qlonglong timeRemaining);

    /*!
      Moves the dialog to idle state. Reason for the state to be passed to cause parameter.
      Dialog shows a resume button in case allowResume is set to true.
     */
    void showIdle(const QString &cause, bool allowResume = true);

    /*!
      Download dialog moves to the failure state. Here the causeForFailure is the
      reason for failure and is shown in the place of progress bar.
      Also, "Retry" and "Cancel" button are shown.
     */
    void showFailure(const QString &causeForFailure);

    /*!
      Download dialog moves to completed state. Downloaded time is shown in the dialog.
     */
    void showCompleted(const QDateTime &downloadedTime);

    /// \todo or do we choose this method?
    // void showCompleted(const QString &message);

signals:
    // Emitted in case user chooses to resume a paused download
    void resumeClicked();
    // Emitted in case user clicks on cancel button
    void cancelClicked();
    // Emitted in case user clicks on retry button
    void retryClicked();
    // Emitted in case user clicks on open button
    void openClicked();

private slots:
    void handleOpenCancelClick();
    void handleResumeRetryClick();

private:
    void removeAllButtons();
    void addOpenCancelButton(const QString &string);
    void addResumeRetryButton(const QString &string);

private:
    Q_DECLARE_PRIVATE(FeedDownloadDialog);
    Q_DISABLE_COPY(FeedDownloadDialog);
    FeedDownloadDialogPrivate *d_ptr;
};

#endif // FEEDDOWNLOADDIALOG_H
